# EmapicConsul

This gem was developed during the [Jornadas de Inteligencia Colectiva para la Democracia](http://medialab-prado.es/article/madrid-inteligencia-colectiva-para-la-democracia).

This gem was used in the [project](http://medialab-prado.es/article/inteligencia-colectiva-para-la-democracia-proyectos-seleccionados) which developed the [consul](https://github.com/consul/consul) + [emapic](https://github.com/Emapic) integration to provide consul with new functionalities such as proposal and votes geolocation.

## Installation

Add this line to your application's Gemfile:

```ruby
gem 'emapic_consul'
```

And then execute:

```
$ bundle
```

Or install it yourself as:

```
$ gem install emapic_consul
```

## Setup

You need to add the following environment variables to your `.bash_profile` (if
you are in *macOS*). An example setup might look like the following:

```bash
# ~/.bash_profile
export EMAPIC_HOST="192.168.1.199:80"
export EMAPIC_LOGIN="example_login"
export EMAPIC_KEY="example_key"
export EMAPIC_SECRET="example_secret"
```

## Usage

To register proposal creation inside Emapic you need to follow the consul customization
[guide](https://github.com/consul/consul/blob/master/CUSTOMIZE_ES.md) to modify
existing models. You'll need to create a custom `Proposal` model with the following
code:

```ruby
# /app/models/custom/proposal.rb
require_dependency Rails.root.join('app', 'models', 'proposal').to_s

class Proposal < ActiveRecord::Base
  after_create :notify_emapic

  def notify_emapic
    EmapicConsul::Api.create_location_group(self)
  end
end
```

The approach to register proposal votes in Emapic is totally different since
the `Vote` class is not directly instantiated by consul but it's done inside the
`ActsAsVotable::Vote` clas of the [`acts_as_votable`](https://github.com/ryanto/acts_as_votable)
gem.

Probably the best way to handle this is to *monkey-patch* this class during the
rails boot, but I wasn't able to do this with my current knowledge of rails, so
I opted for a different approach.

To register proposal voting in Emapic you need to modify the original
`Proposal.register_vote` method, adding the following line:

```ruby
EmapicConsul::Api.vote_location_group(user, self)
```

So its code ends up looking like this:

```ruby
# /app/models/proposal.rb
class Proposal
  def register_vote(user, vote_value, lat, lng)
    if votable_by?(user) && !archived?
      vote_by(voter: user, vote: vote_value)
      EmapicConsul::Api.vote_location_group(self, nil, lat, lng) unless (lat.nil? || lng.nil?)
    end
  end
end
```

Modify the `ProposalsController.vote` method so it looks like the following:

```ruby
def vote
    @proposal.register_vote(current_user, 'yes', params[:lat], params[:lng])
    set_proposal_votes(@proposal)
  end
```

## Contributing

Bug reports and pull requests are welcome on GitHub at https://github.com/amiedes/emapic_consul.

## License

The gem is available as open source under the terms of the [MIT License](http://opensource.org/licenses/MIT).
